<?php

namespace Drupal\media_pixabay\Plugin\EntityBrowser\Widget;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Session\AccountProxy;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\entity_browser\WidgetBase;
use Drupal\entity_browser\WidgetValidationManager;
use Drupal\file\Plugin\Field\FieldType\FileItem;
use Drupal\media_pixabay\Api\PixabayApiServiceInterface;
use GuzzleHttp\Exception\GuzzleException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Uses a view to provide entity listing in a browser's widget.
 *
 * @EntityBrowserWidget(
 *   id = "pixabay",
 *   label = @Translation("Pixabay"),
 *   description = @Translation("Adds a Pixabay widget where images from Pixabay can be added as media to entities (use entity reference to media on entity fields). Only one Pixabay widget per entity browser is supported."),
 *   auto_select = FALSE
 * )
 */
class Pixabay extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   *   Renderer.
   */
  protected Renderer $renderer;

  /**
   * Cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   *   Cache.
   */
  protected CacheBackendInterface $cache;

  /**
   * The media Pixabay API key config.
   *
   * @var string
   */
  protected string $pixabayKeyConfig;

  /**
   * Error logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  protected LoggerChannelFactory $loggerFactory;

  /**
   * Current user service.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected AccountProxy $currentUser;

  /**
   * The "file_system" service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected Token $token;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Pixabay Api service.
   *
   * @var \Drupal\media_pixabay\Api\PixabayApiServiceInterface
   */
  protected PixabayApiServiceInterface $pixabayApiService;

  /**
   * Upload constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   Event dispatcher service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\entity_browser\WidgetValidationManager $validation_manager
   *   The Widget Validation Manager service.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   Renderer.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   Cache.
   * @param \Drupal\Core\Config\ImmutableConfig $config_factory
   *   Pixabay API key config.
   * @param \Drupal\Core\Logger\LoggerChannelFactory $logger
   *   Error logger.
   * @param \Drupal\Core\Session\AccountProxy $current_user
   *   Current user.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   File system.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Utility\Token $token
   *   The token service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\media_pixabay\Api\PixabayApiServiceInterface $pixabay_api_service
   *   The Pixabay Api service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EventDispatcherInterface $event_dispatcher,
    EntityTypeManagerInterface $entity_type_manager,
    WidgetValidationManager $validation_manager,
    Renderer $renderer,
    CacheBackendInterface $cache,
    ImmutableConfig $config_factory,
    LoggerChannelFactory $logger,
    AccountProxy $current_user,
    FileSystemInterface $file_system,
    ModuleHandlerInterface $module_handler,
    Token $token,
    MessengerInterface $messenger,
    PixabayApiServiceInterface $pixabay_api_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $event_dispatcher, $entity_type_manager, $validation_manager);
    $this->renderer = $renderer;
    $this->cache = $cache;
    $this->pixabayKeyConfig = $config_factory->get('media_pixabay_api');
    $this->loggerFactory = $logger;
    $this->currentUser = $current_user;
    $this->fileSystem = $file_system;
    $this->moduleHandler = $module_handler;
    $this->token = $token;
    $this->messenger = $messenger;
    $this->pixabayApiService = $pixabay_api_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('event_dispatcher'),
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.entity_browser.widget_validation'),
      $container->get('renderer'),
      $container->get('cache.default'),
      $container->get('config.factory')->get('media_pixabay.admin.config'),
      $container->get('logger.factory'),
      $container->get('current_user'),
      $container->get('file_system'),
      $container->get('module_handler'),
      $container->get('token'),
      $container->get('messenger'),
      $container->get('media_pixabay.api_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'upload_location' => 'public://Pixabay/[PIXABAY_SEARCH_TERM]/',
      'submit_text' => $this->t('Select entities'),
      'search_text' => $this->t('Pixabay search'),
      'search_placeholder' => $this->t('Enter search terms...'),
      'extensions' => '',
      'image_size' => '_640',
      'media_type' => NULL,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['search_placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search bar text'),
      '#default_value' => $this->configuration['search_placeholder'],
    ];

    $form['search_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search button text'),
      '#default_value' => $this->configuration['search_text'],
    ];

    $form['upload_location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Upload location'),
      '#default_value' => $this->configuration['upload_location'],
      '#description' => $this->t('[PIXABAY_SEARCH_TERM] token will be replaced with the search term.'),
    ];

    $form['extensions'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Allowed extensions'),
      '#description' => $this->t('The allowed extensions are loaded from the selected media type.'),
      '#default_value' => $this->configuration['extensions'],
      '#element_validate' => [[FileItem::class, 'validateExtensions']],
      '#prefix' => '<div id="pixabay-allowed-extensions">',
      '#suffix' => '</div>',
    ];

    $form['image_size'] = [
      '#type' => 'select',
      '#title' => 'Select image size (Width)',
      '#options' => [
        '_180' => $this->t('180'),
        '_340' => $this->t('340'),
        '_640' => $this->t('640'),
        '_1280' => $this->t('1280'),
        '_1920' => $this->t('1920'),
        'original' => $this->t('Original image'),
      ],
      '#default_value' => $this->configuration['image_size'],
      '#description' => $this->t('Select the wanted image size. Original and fullHD(1920) Images only work with full API access and will default to 640 if not available.'),
    ];

    $form['identifier'] = [
      '#type' => 'hidden',
      '#value' => 'pixabay-widget',
    ];

    if ($this->moduleHandler->moduleExists('token')) {
      $form['token_help'] = [
        '#theme' => 'token_tree_link',
        '#token_types' => ['file'],
      ];
      $form['upload_location']['#description'] = $this->t('You can use tokens in the upload location ([PIXABAY_SEARCH_TERM] token will be replaced with the search term).');
    }

    try {
      $mediaTypeOptions = [];
      $mediaTypes = $this
        ->entityTypeManager
        ->getStorage('media_type')
        ->loadByProperties(['source' => 'image']);
      foreach ($mediaTypes as $mediaType) {
        $mediaTypeOptions[$mediaType->id()] = $mediaType->label();
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('media_pixabay')
        ->alert($e->getMessage());
    }

    if (empty($mediaTypeOptions)) {
      $url = Url::fromRoute('entity.media_type.add_form')->toString();
      $form['media_type'] = [
        '#type' => 'select',
        '#title' => $this->t('Media type'),
        '#default_value' => $this->configuration['media_type'],
        '#required' => TRUE,
        '#description' => $this->t("You don't have a media type with the Image source field. You should
            <a href='@link' target='_blank'>create one.</a>", ['@link' => $url]),
      ];
    }
    else {
      $form['media_type'] = [
        '#type' => 'select',
        '#title' => $this->t('Media type'),
        '#default_value' => $this->configuration['media_type'],
        '#options' => $mediaTypeOptions,
        '#required' => TRUE,
        '#ajax' => [
          'callback' => [$this, 'extensionsAjaxCallback'],
          'disable-refocus' => FALSE,
          'wrapper' => 'pixabay-allowed-extensions',
          'event' => 'change',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Verifying entry...'),
          ],
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  public function getForm(array &$original_form, FormStateInterface $form_state, array $additional_widget_parameters): array {
    $form['#attached']['library'][] = 'core/drupal.ajax';
    $form['#attached']['library'][] = 'media_pixabay/media-pixabay-js-library';
    $form['#attached']['library'][] = 'media_pixabay/media-pixabay-css-library';

    try {
      /** @var \Drupal\media\MediaTypeInterface $mediaType */
      $mediaType = $this->entityTypeManager
        ->getStorage('media_type')
        ->load($this->configuration['media_type']);
      if (!$this->configuration['media_type'] || empty($mediaType)) {
        return ['#markup' => $this->t('The media type is not configured correctly.')];
      }
      if ($mediaType->getSource()->getPluginId() != 'image') {
        return ['#markup' => $this->t('The configured media type is not using the image plugin.')];
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('media_pixabay')
        ->alert($e->getMessage());
    }

    $form['wrapper'] = [
      '#type' => 'container',
      '#prefix' => '',
      '#attributes' => ['id' => 'pixabay-wrapper'],
    ];

    $form['wrapper']['pixabay_logo'] = [
      '#prefix' => '<div class="pixabay-logo-widget">',
      '#suffix' => '</div>',
      '#markup' => '<a href="https://pixabay.com/" target="_blank">
                        <img src="https://pixabay.com/static/img/public/leaderboard_a.png" alt="Pixabay">
                    </a>',
    ];

    /** @var \Drupal\media\Entity\MediaType $mediaType */
    $mediaType = $this->entityTypeManager
      ->getStorage('media_type')
      ->load($this->configuration['media_type']);
    if ($mediaType != NULL) {
      /** @var \Drupal\field\Entity\FieldConfig $fieldInstance */
      $fieldInstance = $this->entityTypeManager
        ->getStorage('field_config')
        ->load('media.' . $this->configuration['media_type'] . '.' . $mediaType->getSource()->getConfiguration()['source_field']);
      $allowedExtensionsMt = explode(' ', $fieldInstance->get('settings')['file_extensions']);
      $allowedExtensionsWidget = explode(' ', $this->getConfiguration()['settings']['extensions']);
      if ($allowedExtensionsMt != $allowedExtensionsWidget && array_diff($allowedExtensionsWidget, $allowedExtensionsMt)) {
        $this->messenger->addMessage($this->t('The allowed extensions in the selected media type and in the Pixabay widget do not match. Please update the widget.'), 'error');
      }
    }

    $form['wrapper']['search_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Pixabay search'),
      '#attributes' => [
        'placeholder' => $this->configuration['search_placeholder'],
        'id' => 'pixabay-search-input',
      ],
    ];

    $form['wrapper']['actions'] = [
      [
        '#type' => 'actions',
        'search' => [
          '#type' => 'button',
          '#name' => 'pixabaySearch',
          '#value' => $this->configuration['search_text'],
          '#button_type' => 'primary',
          '#attributes' => ['id' => 'pixabay-search'],
        ],
      ],
      [
        '#type' => 'actions',
        'submit' => [
          '#type' => 'submit',
          '#name' => 'pixabaySubmit',
          '#value' => $this->configuration['submit_text'],
          '#eb_widget_main_submit' => TRUE,
          '#attributes' => [
            'class' => ['is-entity-browser-submit'],
            'id' => ['pixabay-submit-entities'],
          ],
          '#button_type' => 'primary',
        ],
      ],
    ];

    $form['wrapper']['pictures'] = [
      '#type' => 'container',
      '#prefix' => '<div id="result-images">',
      '#suffix' => '</div>',
      '#tree' => TRUE,
      '#weight' => 10,
    ];

    $triggeringElement = $form_state->getTriggeringElement()['#array_parents'] ?? FALSE;
    if ($form_state->get('images') === NULL || in_array('widget_selector', $triggeringElement, TRUE)) {
      $form_state->set('images', []);
    }
    elseif ($form_state->get('images')->hits) {
      $this->buildImages($form_state->get('images')->hits, $form);
    }

    return $form;
  }

  /**
   * Sets the #checked property when rebuilding form.
   *
   * Every time when we rebuild we want all checkboxes to be unchecked.
   *
   * @param array $element
   *   An associative array containing the properties and children of the
   *   generic form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $complete_form
   *   The complete form.
   *
   * @return array
   *   Returns the processed elements array.
   *
   * @see \Drupal\Core\Render\Element\Checkbox::processCheckbox()
   */
  public static function processCheckbox(array &$element, FormStateInterface $form_state, array &$complete_form): array {
    if ($form_state->isRebuilding()) {
      $element['#checked'] = FALSE;
    }
    else {
      $value = $element['#value'];
      $returnValue = $element['#return_value'];
      if ($value === TRUE || $value === FALSE || $value === 0) {
        $element['#checked'] = (bool) $value;
      }
      else {
        $element['#checked'] = ($value['id'] === $returnValue['id']);
      }
    }
    return $element;
  }

  /**
   * Prepares entities to be selected by the entity browser.
   *
   * Creates files from images and saves them to the Pixabay directory based on
   * date of search (folders by months per year) and search term. Then media
   * entities are created and saved.
   *
   * @param array $form
   *   Complete form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return \Drupal\Core\Entity\EntityInterface[]
   *   Array of saved media entities.
   */
  protected function prepareEntities(array $form, FormStateInterface $form_state): array {
    $media = [];
    foreach ($form_state->getValue('pictures') as $key => $item) {
      if (!empty($item['checked'])) {
        $searchTerm = $form_state->getValue('search_key');
        $path = str_replace('[PIXABAY_SEARCH_TERM]', $searchTerm, $this->token
          ->replace($this->configuration['upload_location']));
        if ($this->fileSystem->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY)) {
          /** @var \Drupal\file\FileInterface $file */
          $file = system_retrieve_file($item['checked']['url'], $path, TRUE);
          if ($file !== FALSE) {
            try {
              /** @var \Drupal\media\MediaTypeInterface $mediaType */
              $mediaType = $this->entityTypeManager
                ->getStorage('media_type')
                ->load($this->configuration['media_type']);
              /** @var \Drupal\media\Entity\Media[] $media */
              $media[$key] = $this->entityTypeManager
                ->getStorage('media')
                ->create([
                  'bundle' => $this->configuration['media_type'],
                  'uid' => $this->currentUser->id(),
                  'status' => TRUE,
                  $mediaType->getSource()
                    ->getConfiguration()['source_field'] => [
                      'target_id' => $file->id(),
                      'title' => 'Pixabay: ' . $searchTerm,
                      'alt' => $item['checked']['tags'],
                    ],
                ]);
              $media[$key]->setName('Pixabay: ' . $searchTerm)
                ->save();
              $media = array_values($media);
            }
            catch (\Exception $e) {
              $this->loggerFactory->get('media_pixabay')
                ->alert($e->getMessage());
            }
          }
        }
      }
    }
    return $media;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function validate(array &$form, FormStateInterface $form_state): void {
    if ($form_state->getTriggeringElement()['#name'] === 'pixabaySubmit') {
      $mediaType = $this->configuration['media_type'];
      /** @var \Drupal\media\MediaTypeInterface $mediaTypeLoaded */
      $mediaTypeLoaded = $this->entityTypeManager
        ->getStorage('media_type')
        ->load($mediaType);
      $sourceField = $mediaTypeLoaded->getSource()
        ->getConfiguration()['source_field'];
      /** @var \Drupal\field\Entity\FieldConfig $mediaTypeFieldConfig */
      $mediaTypeFieldConfig = $this->entityTypeManager
        ->getStorage('field_config')
        ->load('media.' . $mediaType . '.' . $sourceField);
      $mediaTypeExtensions = $mediaTypeFieldConfig->getSetting('file_extensions');
      foreach ($form_state->getValue('pictures') as $key => $item) {
        if (!empty($item['checked'])) {
          array_push($form['widget']['wrapper']['pictures'][$key]['#attributes']['class'], 'pixabay-selected-image');
          $fileExtension = pathinfo($item['checked']['url'], PATHINFO_EXTENSION);
          if (strpos($this->configuration['extensions'], $fileExtension) === FALSE) {
            $form_state->setError($form['widget']['wrapper']['pictures'][$key],
              $this->t("Extension @extension doesn't match the allowed extensions set in the widget configuration.", ['@extension' => $fileExtension]));
            array_push($form['widget']['wrapper']['pictures'][$key]['#attributes']['class'], 'pixabay-configuration-error');
          }
          elseif (strpos($mediaTypeExtensions, $fileExtension) === FALSE) {
            $form_state->setError($form['widget']['wrapper']['pictures'][$key],
              $this->t("Extension @extension doesn't match the allowed extensions set in the media type.", ['@extension' => $fileExtension]));
            array_push($form['widget']['wrapper']['pictures'][$key]['#attributes']['class'], 'pixabay-configuration-error');
          }
        }
      }
    }
    if ($form_state->getTriggeringElement()['#name'] === 'pixabaySearch') {
      if (empty($this->pixabayKeyConfig)) {
        $form_state->setError($form['widget']['wrapper']['search_key'], $this->t('Please configure your Pixabay API search key.'));
        return;
      }
      $searchTerm = $form_state->getValue('search_key');
      $queryStr = UrlHelper::buildQuery([
        'key' => $this->pixabayKeyConfig,
        'q' => $searchTerm,
        'image_type' => 'image,illustration',
        'per_page' => 100,
      ]);
      if (!empty($searchTerm)) {
        // Check if we have cached results for current search term.
        $cid = 'media_pixabay:' . $searchTerm;
        if ($cache = $this->cache->get($cid)) {
          $pixabayReturn = $cache->data;
          $form_state->set('images', $pixabayReturn);
        }
        else {
          try {
            $pixabayReturn = $this->pixabayApiService
              ->apiCall($queryStr);
            if ($pixabayReturn->hits) {
              // Set the request cache to 24 hours, according to Pixabay API
              // rules.
              $this->cache->set($cid, $pixabayReturn, time() + 24 * 60 * 60, ['pixabay:' . $searchTerm]);
              $form_state->set('images', $pixabayReturn);
            }
            else {
              $form['widget']['wrapper']['pictures'] = NULL;
              $form_state->setError($form['widget']['wrapper']['search_key'],
                $this->t('Current search provided no results, please enter a valid search term.'));
            }
          }
          catch (GuzzleException $e) {
            $form_state->setError($form['widget']['wrapper']['search_key'],
              $this->t('The validation of "@url" failed with error "@error" (HTTP code @code).', [
                '@url' => UrlHelper::filterBadProtocol('https://pixabay.com/api/?' . $queryStr),
                '@error' => $e->getMessage(),
                '@code' => $e->getCode(),
              ]));
          }
        }
      }
      else {
        $form['widget']['wrapper']['pictures'] = NULL;
        $form_state->setError($form['widget']['wrapper']['search_key'], 'Please enter a search term.');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submit(array &$element, array &$form, FormStateInterface $form_state): void {
    if (!empty($form_state->getTriggeringElement()['#eb_widget_main_submit'])) {
      $images = $this->prepareEntities($form, $form_state);
      $this->selectEntities($images, $form_state);
    }
  }

  /**
   * Builds images.
   *
   * @param array $content
   *   Resulting images.
   * @param array $form
   *   Form.
   */
  private function buildImages(array $content, array &$form): void {
    foreach ($content as $key => $pixabayImage) {
      $imageSize = $this->configuration['image_size'];
      $url = $pixabayImage->webformatURL;

      // Check if image size selected is original or fullHD which are for
      // full API access then check if the user has full API access, if not
      // revert to image size 640.
      switch ($imageSize) {
        case 'original':
          if (array_key_exists('imageURL', $pixabayImage)) {
            $url = $pixabayImage->imageURL;
          }
          break;

        case '_1920':
          if (array_key_exists('fullHDURL', $pixabayImage)) {
            $url = $pixabayImage->fullHDURL;
          }
          break;

        case '_1280':
          $url = $pixabayImage->largeImageURL;
          break;

        default:
          $tmpUrlExtension = explode('.', $pixabayImage->webformatURL);
          $urlExtension = end($tmpUrlExtension);
          $imageUrl = explode('_', $pixabayImage->webformatURL);
          array_pop($imageUrl);
          $url = implode('', $imageUrl) . $imageSize . '.' . $urlExtension;
          break;
      }

      $form['wrapper']['pictures'][$key] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => ['pixabay-image-wrapper'],
        ],
        '#tree' => TRUE,
      ];

      $form['wrapper']['pictures'][$key]['checked'] = [
        '#type' => 'checkbox',
        '#attributes' => [
          'class' => ['pixabay-image-check'],
        ],
        '#process' => [
          ['\Drupal\media_pixabay\Plugin\EntityBrowser\Widget\Pixabay',
            'processCheckbox',
          ],
          ['\Drupal\Core\Render\Element\Checkbox', 'processAjaxForm'],
        ],
        '#return_value' => [
          'url' => $url,
          'preview' => $pixabayImage->previewURL,
          'id' => $pixabayImage->id,
          'tags' => $pixabayImage->tags,
        ],
      ];

      $form['wrapper']['pictures'][$key]['image'] = [
        '#markup' => '<img src="' . $pixabayImage->previewURL . '"
            width="' . $pixabayImage->previewWidth . '" height="' . $pixabayImage->previewHeight . '"
            alt="Pixabay image" class="pixabay-image">',
      ];

    }
  }

  /**
   * Adjusts allowed extensions for the selected media type.
   *
   * @param array $form
   *   The complete form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array|void
   *   Form array with changed data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function extensionsAjaxCallback(array &$form, FormStateInterface $form_state) {
    $mediaType = $form_state->getValue('table')[$this->uuid()]['form']['media_type'];
    if ($mediaType !== '_none' && $mediaType !== '') {
      /** @var \Drupal\media\Entity\MediaType $mediaTypeLoaded */
      $mediaTypeLoaded = $this->entityTypeManager
        ->getStorage('media_type')
        ->load($mediaType);
      /** @var \Drupal\field\Entity\FieldConfig $fieldInstance */
      $fieldInstance = $this->entityTypeManager->getStorage('field_config')
        ->load('media.' . $mediaType . '.' . $mediaTypeLoaded->getSource()
          ->getConfiguration()['source_field']);
      $extensions = $fieldInstance->get('settings')['file_extensions'];
      $this->configuration['extensions'] = $extensions;
      $this->configuration['media_type'] = $mediaType;
      $uuid = '';
      if (empty($this->uuid())) {
        foreach ($form_state->getValue('table') as $key => $value) {
          if (is_array($value) && isset($value['form']['identifier']) && $value['form']['identifier'] == 'pixabay-widget') {
            $uuid = $key;
          }
        }
      }
      else {
        $uuid = $this->uuid();
      }
      $form['widgets']['table'][$uuid]['form']['extensions']['#value'] = $extensions;
      return $form['widgets']['table'][$uuid]['form']['extensions'];
    }
  }

  /**
   * {@inheritDoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $mediaType = $form_state->getValue('table')[$this->uuid()]['form']['media_type'];
    if ($mediaType === '_none' || $mediaType === '') {
      $form_state->setErrorByName('table][' . $this->uuid() . '][form][media_type',
        $this->t('Please select a media type.'));
    }
    else {
      /** @var \Drupal\media\Entity\MediaType $mediaTypeLoaded */
      $mediaTypeLoaded = $this->entityTypeManager
        ->getStorage('media_type')
        ->load($mediaType);
      if ($mediaTypeLoaded != NULL) {
        /** @var \Drupal\field\Entity\FieldConfig $fieldInstance */
        $fieldInstance = $this->entityTypeManager
          ->getStorage('field_config')
          ->load('media.' . $mediaType . '.' . $mediaTypeLoaded->getSource()
            ->getConfiguration()['source_field']);
        $allowedExtensions = explode(" ", $fieldInstance->get('settings')['file_extensions']);
        $submittedExtensionsArray = explode(" ", $form_state->getValue('table')[$this->uuid()]['form']['extensions']);
        if ($allowedExtensions != $submittedExtensionsArray && array_diff($submittedExtensionsArray, $allowedExtensions)) {
          $form_state->setErrorByName('table][' . $this->uuid() . '][form][extensions',
            $this->t('The allowed extensions on this widget do not match those that are allowed on the media type @media, they might have been changed.
           Try saving the configuration again.', [
             '@media' => $mediaType,
           ]));
          $this->extensionsAjaxCallback($form, $form_state);
        }
      }
    }
  }

}
