<?php

namespace Drupal\Tests\media_pixabay\Functional;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\media\MediaTypeInterface;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;
use Drupal\user\UserInterface;

/**
 * Tests for the Pixabay API key configuration form.
 *
 * @group Media Pixabay
 *
 * @package Drupal\Tests\media_pixabay\Functional
 */
class MediaPixabayApiKeyConfigurationTest extends BrowserTestBase {

  use MediaTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'node',
    'media',
    'media_pixabay',
    'entity_browser',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * User to test the Pixabay media forms.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $user;

  /**
   * Media type entity.
   *
   * @var \Drupal\media\MediaTypeInterface
   */
  protected MediaTypeInterface $mediaType;

  /**
   * Media storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $mediaStorage;

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  public function setUp() : void {
    parent::setUp();
    /** @var \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager */
    $entityTypeManager = $this->container->get('entity_type.manager');
    $this->mediaStorage = $entityTypeManager->getStorage('media');
    $this->mediaType = $this->createMediaType('image');
  }

  /**
   * Test Pixabay API key configuration form.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testApiKeyConfigurationForm() {
    // Test successful access.
    $this->assertApiKeyConfigurationStatusCode([
      'administer pixabay settings',
    ], 200);

    // Test access denied.
    $this->assertApiKeyConfigurationStatusCode([], 403);

    // Test empty required field and successful API key form submit scenarios.
    $this->loginCreateUser([
      'administer pixabay settings',
      'administer site configuration',
    ]);
    $this->assertApiKeyConfigurationSubmitMarkup('admin/config/media/pixabay', [], 'Pixabay API key field is required.');
    $this->assertApiKeyConfigurationSubmitMarkup('admin/config/media/pixabay', [
      'media_pixabay_api' => '4321testApiKey1234',
    ], 'The configuration options have been saved.');
  }

  /**
   * User login function.
   *
   * @param array $permissions
   *   User permissions.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function loginCreateUser(array $permissions): void {
    $this->drupalLogin($this->drupalCreateUser($permissions));
  }

  /**
   * Visits the Pixabay API configuration and assert the a status code.
   *
   * @param array $permissions
   *   User permissions.
   * @param int $expectedStatusCode
   *   Expected status code when visiting revisions list.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function assertApiKeyConfigurationStatusCode(array $permissions, int $expectedStatusCode): void {
    $this->loginCreateUser($permissions);
    $this->drupalGet('admin/config/media/pixabay');
    $this->assertSession()->statusCodeEquals($expectedStatusCode);
  }

  /**
   * Submits the Pixabay API configuration form and assert the status message.
   *
   * @param string $path
   *   Form path.
   * @param array $edit
   *   Form fields to be filled with values.
   * @param string $statusMessage
   *   Expected status message when submitting the form.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  private function assertApiKeyConfigurationSubmitMarkup($path, array $edit, string $statusMessage): void {
    $this->drupalGet($path);
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains($statusMessage);
  }

}
